@@ -114,11 +114,11 @@ typedef struct rx_hash_item * (*rx_alloc_hash_item)();
 typedef void (*rx_free_hash_item)();
 #endif
 
-struct rx_hash_rules 
+struct rx_hash_rules
 {
   rx_hash_eq eq;
   rx_alloc_hash hash_alloc;
-  rx_free_hash free_hash;	/* oh, shut up don */
+  rx_free_hash free_hash;
   rx_alloc_hash_item hash_item_alloc;
   rx_free_hash_item free_hash_item;
 };
@@ -140,8 +140,17 @@ struct rx_inx
  * converted on demand to a more efficient nfa.  This is for the low level interface.
  * The high-level interface incloses this in a `struct re_pattern_buffer'.
  */
-
 struct rx_cache;
+#ifdef __STDC__
+struct rx_se_list;
+struct rx;
+typedef int (*rx_se_list_order) (struct rx *,
+				 struct rx_se_list *, struct rx_se_list *);
+#else
+typedef int (*rx_se_list_order) ();
+#endif
+
+struct rx_superset;
 
 struct rx
 {
@@ -187,6 +196,11 @@ struct rx
   void ** instruction_table;
   struct rx_nfa_state *nfa_states;
   struct rx_nfa_state *start;
+
+  /* This orders the search through super-nfa paths. */
+  rx_se_list_order se_list_cmp;
+
+  struct rx_superset * start_set;
 };
 
 /* An RX NFA may contain epsilon edges labeled with side effects.
@@ -206,7 +220,7 @@ struct rx
 
 typedef void * rx_side_effect;
 
-struct rx_se_list 
+struct rx_se_list
 {
   rx_side_effect car;
   struct rx_se_list * cdr;
@@ -396,6 +410,13 @@ struct rx_superset
   /* If the corresponding superstate exists: */
   struct rx_superstate * superstate;
 
+  /* If this is a starting state (as built by re_search_2)
+   * this points to the `struct rx'.  The memory for these objects
+   * is typed -- so even after they are freed it is safe to look
+   * at this field (to check, in fact, if this was freed.)
+   */
+  struct rx * starts_for;
+
   struct rx_hash_item hash_item;
 };
 
@@ -417,6 +438,9 @@ struct rx_super_edge
 /* A superstate is a set of nfa states (RX_SUPERSET) along
  * with a transition table.  Superstates are built on demand and reclaimed
  * without warning.  To protect a superstate, use LOCK_SUPERSTATE.
+ *
+ * Joe Keane thought of calling these superstates and several people
+ * have commented on what a good name it is for what they do. 
  */
 struct rx_superstate
 {
@@ -434,7 +458,7 @@ struct rx_superstate
 
 struct rx_distinct_future
 {
-  struct rx_distinct_future * next_same_super_edge;
+  struct rx_distinct_future * next_same_super_edge[2];
   struct rx_distinct_future * next_same_dest;
   struct rx_distinct_future * prev_same_dest;
   struct rx_superstate * present;	/* source state */
@@ -979,11 +1003,6 @@ extern void regfree ();
    * it requires an argument to tell which group is being matched.
    * 
    * The integer name of a complex effect is an index into rxb->se_params.
-   *
-   * The ordering of these effects is significant when searching.  re_se_win
-   * should always remain the most negative side effect.  The other values
-   * are less critical but the goal is that the shorter path to a win
-   * should always be the more negative.
    */
  
   RX_DEF_SE(1, re_se_try, = -1)		/* Epsilon from start state */
@@ -1015,21 +1034,20 @@ extern void regfree ();
   RX_DEF_SE(1, re_se_dollar, = re_se_endbuf - 1)
 
   /* Simple effects: */
-  RX_DEF_SE(1, re_se_fail, = re_se_dollar - 1) /* These are rare, = so order doesn't matter. */
-  RX_DEF_SE(1, re_se_win, = re_se_fail - 1)			/* Epsilon to a final state */
+  RX_DEF_SE(1, re_se_fail, = re_se_dollar - 1)
 
   /* Complex effects.  These are used in the 'se' field of 
-   * an RX_DEF_SE(re_se_params structure.  Indexes into the se array
+   * a struct re_se_params.  Indexes into the se array
    * are stored as instructions on nfa edges.
-   *
-   * Don't be confused that these are declared here.  They never occur in the
-   * op field of a struct rx_inx.
    */
-  RX_DEF_CPLX_SE(1, re_se_lparen, = 0)
+  RX_DEF_CPLX_SE(1, re_se_win, = 0)
+  RX_DEF_CPLX_SE(1, re_se_lparen, = re_se_win + 1)
   RX_DEF_CPLX_SE(1, re_se_rparen, = re_se_lparen + 1)
   RX_DEF_CPLX_SE(0, re_se_backref, = re_se_rparen + 1)
   RX_DEF_CPLX_SE(0, re_se_iter, = re_se_backref + 1) 
   RX_DEF_CPLX_SE(0, re_se_end_iter, = re_se_iter + 1)
+  RX_DEF_CPLX_SE(0, re_se_tv, = re_se_end_iter + 1)
+
 #endif
 
 #endif
