@@ -75,7 +75,7 @@ bcopy (from, to, len)
 #endif
 #endif
 
-char *version_string = "GNU sed version 2.03";
+char *version_string = "GNU sed version 2.05";
 
 /* Struct vector is used to describe a chunk of a compiled sed program.  
  * There is one vector for the main program, and one for each { } pair,
@@ -124,7 +124,8 @@ enum addr_types
   addr_is_null = 0,
   addr_is_num = 1,
   addr_is_regex = 2,
-  addr_is_last = 3
+  addr_is_last = 3,
+  addr_is_mod = 4
 };
 
 struct addr
@@ -132,6 +133,7 @@ struct addr
   int addr_type;
   struct re_pattern_buffer *addr_regex;
   int addr_number;
+  int modulo, offset;
 };
 
 
@@ -210,16 +212,15 @@ struct line
 };
 
 /* This structure holds information about files opend by the 'r', 'w',
-   and 's///w' commands.  In paticular, it holds the FILE pointer to
-   use, the file's name, a flag that is non-zero if the file is being
-   read instead of written. */
+   and 's///w' commands.  In paticular, it holds the FILE pointers to
+   use, the file's name. */
 
 #define NUM_FPS	32
 struct
   {
-    FILE *phile;
+    FILE *for_read;
+    FILE *for_write;
     char *name;
-    int readit;
   }
 
 file_ptrs[NUM_FPS];
@@ -381,7 +382,7 @@ main (argc, argv)
 
   /* see regex.h */
   re_set_syntax (RE_SYNTAX_POSIX_BASIC);
-  rx_cache_bound = 10000;	/* Consume memory rampantly. */
+  rx_cache_bound = 4096;	/* Consume memory rampantly. */
 
   myname = argv[0];
   while ((opt = getopt_long (argc, argv, "hne:f:V", longopts, (int *) 0))
@@ -407,6 +408,12 @@ main (argc, argv)
 	  compiled = 1;
 	  break;
 	case 'f':
+	  if (e_strings)
+	    {
+	      compile_string (e_strings);
+	      free (e_strings);
+	      e_strings = 0;
+	    }
 	  compile_file (optarg);
 	  compiled = 1;
 	  break;
@@ -485,8 +492,10 @@ close_files ()
 
   for (nf = 0; nf < NUM_FPS; nf++)
     {
-      if (file_ptrs[nf].phile)
-	fclose (file_ptrs[nf].phile);
+      if (file_ptrs[nf].for_write)
+	fclose (file_ptrs[nf].for_write);
+      if (file_ptrs[nf].for_read)
+	fclose (file_ptrs[nf].for_read);
     }
 }
 
@@ -960,11 +969,28 @@ compile_address (addr)
       num = ch - '0';
       while ((ch = inchar ()) != EOF && isdigit (ch))
 	num = num * 10 + ch - '0';
+      if (ch == '~')
+	{
+	  addr->addr_type = addr_is_mod;
+	  addr->offset = num;
+	  ch = inchar();
+	  num=0;
+	  if (isdigit(ch)) {
+	    num = ch - '0';
+	    while ((ch = inchar ()) != EOF && isdigit (ch))
+	      num = num * 10 + ch - '0';
+	    addr->modulo = num;
+	  }
+	  addr->modulo += (addr->modulo==0);
+	}
+      else
+	{
+	  addr->addr_type = addr_is_num;
+	  addr->addr_number = num;
+	}
       while (ch != EOF && isblank (ch))
 	ch = inchar ();
       savchar (ch);
-      addr->addr_type = addr_is_num;
-      addr->addr_number = num;
       return 1;
     }
   else if (ch == '/' || ch == '\\')
@@ -1171,26 +1197,22 @@ compile_filename (readit)
     {
       if (!file_ptrs[n].name)
 	break;
-      if (!strcmp (file_ptrs[n].name, file_name))
-	{
-	  if (file_ptrs[n].readit != readit)
-	    bad_prog ("Can't open file for both reading and writing");
-	  flush_buffer (b);
-	  return file_ptrs[n].phile;
-	}
     }
   if (n < NUM_FPS)
     {
       file_ptrs[n].name = ck_strdup (file_name);
-      file_ptrs[n].readit = readit;
       if (!readit)
-	file_ptrs[n].phile = ck_fopen (file_name, "w");
+	{
+	  if (!file_ptrs[n].for_write)
+	    file_ptrs[n].for_write = ck_fopen (file_name, "w");
+	}
       else
 	{
-	  file_ptrs[n].phile = ck_fopen (file_name, "r");
+	  if (!file_ptrs[n].for_read)
+	    file_ptrs[n].for_read = fopen (file_name, "r");
 	}
       flush_buffer (b);
-      return file_ptrs[n].phile;
+      return readit ? file_ptrs[n].for_read : file_ptrs[n].for_write;
     }
   else
     {
@@ -1360,6 +1382,7 @@ restart:
 
 	      n = j->v->v_length - j->v_index;
 	      cur_cmd = j->v->v + j->v_index;
+	      vec = j->v;
 	      goto exe_loop;
 	    }
 	  break;
@@ -1676,6 +1699,7 @@ restart:
 
 		  n = j->v->v_length - j->v_index;
 		  cur_cmd = j->v->v + j->v_index;
+		  vec = j->v;
 		  goto exe_loop;
 		}
 	    }
@@ -1683,7 +1707,10 @@ restart:
 
 	case 'w':
 	  if (cur_cmd->x.io_file)
-	    ck_fwrite (line.text, 1, line.length, cur_cmd->x.io_file);
+	    {
+	      ck_fwrite (line.text, 1, line.length, cur_cmd->x.io_file);
+	      fflush (cur_cmd->x.io_file);
+	    }
 	  break;
 
 	case 'x':
@@ -1728,6 +1755,9 @@ match_address (addr)
       return 1;
     case addr_is_num:
       return (input_line_number == addr->addr_number);
+    case addr_is_mod:
+      return ((input_line_number%addr->modulo) == addr->offset);
+
 
     case addr_is_regex:
       {
